﻿// See https://aka.ms/new-console-template for more information

using ENTTEC.Devices;
using ENTTEC.Devices.Data;
using System.IO.Ports;

 var port = new SerialPort("COM7", 9600, Parity.None, 8, StopBits.One);

port.Open();
port.DataReceived += Port_DataReceived;

var widgetParamsRequestData = DmxUsbProUtils
    .CreatePacketForDevice(
        DmxUsbProConstants.GET_WIDGET_PARAMS_LABEL);
port.Write(widgetParamsRequestData, 0, widgetParamsRequestData.Length);

var messageReceived = false;
while(!messageReceived)
{
    await Task.Delay(100).ConfigureAwait(false);
}

Console.WriteLine("Press ENTER to exit");
Console.ReadLine();

void Port_DataReceived(Object sender, SerialDataReceivedEventArgs e)
{
    if (e.EventType == SerialData.Chars)
    {
        var bufferSize = 1024;
        var buffer = new Byte[bufferSize];
        var readData = port.Read(buffer, 0, bufferSize);

        var handler = Fabric.Instance.GetDmxUsbProMessageHandler<DmxUsbProParamsType>();
        handler.HandleMessage(buffer);
        if (handler.MessageReady)
        {
            var message = handler.GetMessage();
            Console.WriteLine("---DMX USB PRO parameters---");
            Console.WriteLine($"Firmware: {message.Firmware.FirmwareMSB}.{message.Firmware.FirmwareLSB}");
            var breakTime = ConvertToUserValue(message.BreakTime);
            Console.WriteLine($"Break time (microseconds): {breakTime}");
            var mab = ConvertToUserValue(message.MaBTime);
            Console.WriteLine($"Mark After break (MAB in microseconds): {mab}");
            Console.WriteLine($"Packet refresh rate (per second): {message.RefreshRate}");
            messageReceived = true;
        }
    }   
}

Int32 ConvertToUserValue(Int32 deviceValue)
{
    var converted = deviceValue * DmxUsbProConstants
        .DmxMultiplicationCoefficient;

    var result = (Int32)converted;

    return result;
}